package com.android.launcher3.shortcuts

import android.content.Context
import android.support.v4.view.ViewCompat
import android.util.AttributeSet
import android.view.Gravity
import android.view.ViewGroup
import com.teslacoilsw.kotlin.forEach
import com.teslacoilsw.launcher.R

/**
 * A GridLayout that is much simpler than GridLayout.
 * Alternatively, a GridLayout for people too dumb (me) to use GridLayout
 *
 * Children must be fixed size and have their layoutparam size set
 */
class DumbGridLayout(context: Context, attrs: AttributeSet) : ViewGroup(context, attrs) {

    private val columnCount: Int
    private val fillDirection: Int

    init {
        val a = context.obtainStyledAttributes(attrs, R.styleable.DumbGridLayout);
        try {
            columnCount = a.getInt(R.styleable.DumbGridLayout_android_columnCount, 1)
            fillDirection = a.getInt(R.styleable.DumbGridLayout_fillDirection, Gravity.START or Gravity.TOP)
        } finally {
            a.recycle()
        }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        var widthSize = MeasureSpec.getSize(widthMeasureSpec)
        val widthMode = MeasureSpec.getMode(widthMeasureSpec)

        var heightSize = MeasureSpec.getSize(heightMeasureSpec)
        val heightMode = MeasureSpec.getMode(heightMeasureSpec)

        if (childCount <= 0) {
            setMeasuredDimension(0, 0)
            return
        }

        val childHeight = getChildAt(0).layoutParams.height
        val childWidth = getChildAt(0).layoutParams.width
        for (i in 0..childCount - 1) {
            val child = getChildAt(i)
            child.measure(MeasureSpec.makeMeasureSpec(childWidth, MeasureSpec.EXACTLY), MeasureSpec.makeMeasureSpec(childHeight, MeasureSpec.EXACTLY))
        }
        if (widthMode != MeasureSpec.EXACTLY)
            widthSize = childWidth * Math.min(columnCount, childWidth)
        if (heightMode != MeasureSpec.EXACTLY)
            heightSize = childHeight * Math.ceil(childCount/columnCount.toDouble()).toInt();
        setMeasuredDimension(widthSize, heightSize)

    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        layoutChildren(l, t, r, b)
    }

    private fun layoutChildren(l: Int, t: Int, r: Int, b: Int) {
        val childWidth = getChildAt(0).measuredWidth
        val childHeight = getChildAt(0).measuredHeight
        val numRows = Math.ceil(childCount/columnCount.toDouble()).toInt()


        val extraWidth = width - columnCount * childWidth
        val numWidthGaps = columnCount - 1
        val widthGapSize = extraWidth / numWidthGaps.toFloat()

        val extraHeight = height - numRows * childHeight
        val numHeightGaps = numRows - 1
        val heightGapSize = extraHeight / numHeightGaps.toFloat()


        var layoutIndex = 0
        var effectiveChildCount = childCount
        forEach { child ->
            val lp = child.layoutParams

            /* We can allow useNextRowIfAtEnd only if we're not in the last row and we're the last
             * column, and there is still space available
             */
            if (lp is DumbGridLayoutParams && lp.useNextRowIfAtEnd &&
                (layoutIndex + 1) % columnCount == 0 && layoutIndex / columnCount < numRows - 1 &&
                effectiveChildCount < columnCount * numRows)
            {
                layoutIndex++
                effectiveChildCount++
            }

            /* Start at bottom right and fill left then up */
            var row =
                if (fillDirection and Gravity.TOP == Gravity.TOP)
                    layoutIndex/columnCount
                else
                    numRows - 1 - layoutIndex/columnCount

            val col =
                if (Gravity.getAbsoluteGravity(fillDirection, ViewCompat.getLayoutDirection(this)) and Gravity.LEFT == Gravity.LEFT)
                    layoutIndex % columnCount
                else
                    columnCount - 1 - layoutIndex % columnCount


            val childLeft = ((col * (childWidth + widthGapSize))).toInt()
            val childTop = ((row * (childHeight + heightGapSize))).toInt()
            child.layout(childLeft, childTop, childLeft + childWidth, childTop + childHeight)
            layoutIndex++
        }
    }
}

class DumbGridLayoutParams(baseLp: ViewGroup.LayoutParams) : ViewGroup.LayoutParams(baseLp) {
    /* If you have two items next to each other you'd like to pair set this on the first one */
    var useNextRowIfAtEnd: Boolean = false
}